%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%Simulation 4 in the Manuscrpit%%%%%%%
%%% Creating an ANN with hybrid Bayesian method for N=5000 data

tic;
load Sim4DataGenerationN5000.mat

nin = 2;		% Number of inputs.
nhidden = 10;		% Number of hidden units.
nout = 1;		% Number of outputs.

aw1=0.075*ones(1,nin); %Setting hyperparameter values with ARD prior
ab1=0.075;
aw2=0.075;
ab2=0.075;

prior=mlpprior(nin,nhidden,nout,aw1,ab1,aw2,ab2); 
nethmc = mlp(nin, nhidden, nout, 'poisson',prior);
%nethmc = mlpinit(nethmc, 10);

nouter = 20;			% Number of outer loops.
ninner = 5;			% Number of innter loops.
options = zeros(1,18);		% Default options vector.
options(1) = 1;			% This provides display of error values.
options(2) = 1.0e-7;		% Absolute precision for weights.
options(3) = 1.0e-7;		% Precision for objective function.
options(14) = 1000;		% Number of training cycles in inner loop. 

% Train using scaled conjugate gradients, re-estimating alpha and beta.
for k = 1:nouter
  nethmc = netopt(nethmc, options,trainx(:,[1 2]), traint, 'scg');
  [nethmc, gamma] = evidence(nethmc, trainx(:,[1 2]), traint, ninner);
  fprintf(1, '\nRe-estimation cycle %d:\n', k);
  fprintf(1, '  alpha =  %8.5f\n', nethmc.alpha);
  %fprintf(1, '  beta  =  %8.5f\n', net.beta);
  fprintf(1, '  gamma =  %8.5f\n\n', gamma);
end


nsamples = 1000;		% Number of retained samples.

options = foptions;     % Default options vector.
options(1) = 1;		% Switch on diagnostics.
options(5) = 1;		% Use persistence
options(7) = 20;	% Number of steps in trajectory.
options(14) = nsamples;	% Number of Monte Carlo samples returned. 
options(15) = 100;	% Number of samples omitted at start of chain.
options(17) = 0.95;	% Alpha value in persistence
options(18) = 0.0002;	% Step size.

w = mlppak(nethmc);
% Initialise HMC
hmc('state', 42);
[samples, energies] = hmc('neterr', w, options, 'netgrad', nethmc, trainx(:,[1 2]), traint);


 yhmc=zeros(size(testX,1),nsamples);
for k = 1:nsamples
  w2 = samples(k,:);
  net2 = mlpunpak(nethmc, w2);
  yhmc(:,k) = mlpfwd(net2, testX(:,[1 2]));
end
predhmcEvi = mean(yhmc,2);
stdpredhmcEvi=std(yhmc,0,2);
lowerstdhmcEvi=predhmcEvi-stdpredhmcEvi;
upperstdhmcEvi=predhmcEvi+stdpredhmcEvi;

trainlambda=transpose(lambda(1:1600));
testlambda=transpose(lambda(1601:2000));

opt_time=toc; % Storing the execution time

%ANN with hybrid Bayesian approach Prediction Errors
RMSEhybridBayesian=sqrt((sum((predhmcEvi-testlambda).^2))/size(testlambda,1));
MAEhybridBayesian=sum(abs(predhmcEvi-testlambda))/size(testlambda,1);
MPEhybridBayesian=sum(abs((predhmcEvi-testlambda)./testlambda))/size(testlambda,1);
RSEhybridBayesian=(sum((predhmcEvi-testlambda).^2))/(sum((testlambda-mean(testlambda)).^2));

save Sim4hybridBayesian.mat